using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;

using IdentityModel.AspNetCore.AccessTokenManagement;
using IdentityModel.Client;

using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;

using WebApplication.Services;

namespace WebApplication
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers().AddNewtonsoftJson();
            services.Configure<IntegrationApiOptions>(Configuration.GetSection(key: nameof(IntegrationApiOptions)));
            services.Configure<AccessTokenManagementOptions>(Configuration.GetSection(key: nameof(AccessTokenManagementOptions)));
            services.AddAccessTokenManagement();
            services.AddHttpClient<IIntegrationService, IntegrationService>("IntegrationClient", (sp, c) =>
                {
                    var options = sp.GetRequiredService<IOptions<IntegrationApiOptions>>().Value;
                    c.BaseAddress = new Uri(options.BaseUrl);
                })
                .AddClientAccessTokenHandler();
            services.AddOpenApiDocument();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseHttpsRedirection();

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints => { endpoints.MapControllers(); });

            app.UseOpenApi(); // serve documents (same as app.UseSwagger())
            app.UseSwaggerUi3(); // serve Swagger UI
        }
    }
}
